<%--
 * Component: dotcom
 *
 * Description:
 *    Stores DICOM objects resided on other HUBs when cleaning up a study residing on multiple HUBs.
 *
 * Parameters:
 *
 *       cStudy      : Study Instance UID of the study being cleaned up
 *       cSession    : Cleanup session id
 *       cUser       : User initiated cleanup (for logging)
 *       cMove       : true indicates study is being moved from one hub to another
 *       cDelReport  : true indicates to remove reports resided on this HUB
 *       cNumFiles   : Number of files uploaded from other HUBs (number of other HUBs)
 *       cHub<N>     : Device ID of the Nth HUB
 *       cFile<N>    : Objects uploaded from the Nth HUB (expected to be a tar.gz file)
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 *
--%><%@
	page import="java.io.*, java.util.*, java.sql.*, imagemed.imsession.*, imagemed.exception.IMException, imagemed.dotcom.*, imagemed.util.*, imagemed.jsputil.*"

%><%@
	page errorPage="../main/exceptionHandler.jsp"

%><%!

	static final String[] m_env = new String[] { "HOME=" + IMEnvBase.getEnv("HOME") };

	static Syslog m_operLogger = new Syslog("cmh_putobjects", 0, IMEnvBase.getEnv("LOG_OPER_FACILITY"));


	/**
	 * Create log messages on importing objects from other HUB servers when correcting a study resides on multiple HUB servers.
	 *
	 * @param study       Study Instance UID of the study to be corrected
	 * @param user        User performing the correction
	 * @param hubs        Device ID(s) of HUB server(s) objects are being imported from
	 * @param isMove      Invoked from Study Move tool (move from one HUB to another)
	 * @param delReport   Is original report reside on this HUB intended to be deleted
	 * @param error       error message if an error occured during correction (null means OK)
	 */
	static void log(String study, String user, Collection<String> hubs, boolean isMove, boolean delReport, String error) {
		String dbName = null;
		String dbModality = null;
		try {
			ResultSet rs = IMEnv.getDB().createStatement().executeQuery("select * from "+IMEnvBase.getEnv("STUDYTBL")+" where STYIUID='"+study+"'");
			if (rs.first()) {
				dbName = rs.getString("PNAME");
				dbModality = rs.getString("MODALITY");
			}
		}
		catch (SQLException e) {
			Helper.DEBUG("DcAutoForward.runCommand", Helper.throwable2String(e));
		}

		String logMessage = null;
		String logLevel = null;
		int logPriority = Syslog.LOG_INFO;
		if (error == null) {
			logLevel = "INFO";
			if (isMove) {
				logMessage = "study move";
			}
			else {
				logMessage = ( delReport ? "remove" : "keep" ) + " existing report";
			}
		}
		else {
			logLevel = "ERROR";
			logMessage = error;
			logPriority = Syslog.LOG_ERR;
		}

		for (Iterator<String> iter = hubs.iterator(); iter.hasNext(); ) {
			String hub = iter.next();
			
			OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "cmhimport" );
			logEntry.setField( OperLogEntry.FIELD.USER, user );
			logEntry.setField( OperLogEntry.FIELD.STUDY, study );
			logEntry.setField( OperLogEntry.FIELD.PNAME, dbName );
			logEntry.setField( OperLogEntry.FIELD.MODALITY, dbModality );
			logEntry.setField( OperLogEntry.FIELD.HOST, hub );
			logEntry.setField( OperLogEntry.FIELD.MESSAGE, logMessage );
			logEntry.log( m_operLogger, logPriority );

//			m_operLogger.syslog(logPriority, logLevel + " STUDY cmhimport "+ logUser+" - "+study+" "+dbName+" "+dbModality+" "+ logHub +" - - - - - - " + logMessage);
		}
	}


		
	/**
	 * Check if device has an archive configured and sets the ArchiveDevice setting to point to the archive configured on the device
	 *
	 * @param study         Study Instance UID of the study ArchiveDevice to be set for
	 * @param storestate    storestate of the study ArchiveDevice to be set for
	 * @param device        ID of the device to be checked
	 *
	 * @return true if and only if ArchiveDevice setting has been set
	 *
	 * @throws SQLException if an SQL error occurs
	 * @throws IMException if an error occurs
	 */
	static boolean checkAndSetArchiveDeviceTo(String study, StoreState storestate, String deviceId) throws SQLException, IMException {
		DotcomDevice device = DotcomConfig.getConfig().getDevice(deviceId);
		if (device == null)
			return false;

		IMProperties archiveSettings = device.getArchiveSettings();
		String archiveSetting = null == archiveSettings ? null : archiveSettings.getProperty("Archive");
		String archiveDevice = null;
		if ("yes".equals(archiveSetting)) {
			// Set archive device to @Archive-<deviceId>
			archiveDevice="@Archive-" + deviceId;
		}
		else if ("external".equals(archiveSetting)) {
			// Set archive device to @Archive-<externalStore>@<deviceId>
			String externalDev = archiveSettings.getProperty("ExternalStore");
			if (externalDev != null)
				archiveDevice="@Archive-" + externalDev + "@" + deviceId;
		}

		if (archiveDevice == null)
			return false;

		Vector<String> params = new Vector<String>();
		params.add("--archive");
		params.add(archiveDevice);
		storestate.modify(params);
		StudyAttributes.synchronizeAttributes( null, study, null, true );
		return true;
	}


	/**
	 * Checks and fixes archive server setting for a study
	 *
	 * @param study   study to be checked
	 *
	 * @throws SQLException if an SQL error occurs
	 * @throws IMException if an error occurs
	 */
	static void checkAndFixArchiveServer(String study) throws SQLException, IMException {
		StoreState storestate = new StoreState(study);
		storestate.load();

		String currentArchiveServer = storestate.getArchiveServer();
		if (currentArchiveServer == null) {
			// No archive server set for the study, assumed to be OK
			return;
		}
		
		Vector<String> archiveServers = DotcomConfig.getConfig().getArchiveServers();
		if ( archiveServers == null || archiveServers.isEmpty() ) {
			// No archive server configured in the subtree
			// We might want to delete ArchiveServer setting
		}

		for ( String archive : archiveServers ) {
			if ( currentArchiveServer.equals( archive ) ) {
				// The archive server set is in the subtree, assumed to be OK
				// However re-synchronize study attributes toward the cwl server - for it might mismatch
				StudyAttributes.synchronizeAttributes( null, study, null, true );
				return;
			}
		}

		// Check if current server has an archive configured
		if (checkAndSetArchiveDeviceTo(study, storestate, DotcomConfig.selfId())) {
			// Self device has an archive configured
			return;
		}
		
		// Check if owner has an archive configured
		String ownerDevice = storestate.getAttribute("OwnerDevice", "NULL");
		if (!"NULL".equals(ownerDevice)) {
			if (checkAndSetArchiveDeviceTo(study, storestate, ownerDevice)) {
				// Owner device has an archive configured
				return;
			}

			for ( String archive : archiveServers ) {
				if ( ownerDevice.equals( DotcomConfig.getConfig().getServerNameTowards( archive ) ) ) {
					// Owner device is on the route to an archive
					if ( checkAndSetArchiveDeviceTo( study, storestate, archive ) ) {
						return;
					}
				}
			}
		}
		
		// Check if staged devices have archive configured
		Set<String> stagedDevices = storestate.getStagedDevices();
		for (Iterator<String> iter = stagedDevices.iterator(); iter.hasNext(); ) {
			String device = iter.next();
			if (checkAndSetArchiveDeviceTo(study, storestate, device)) {
				// Staged device has an archive configured 
				return;
			}
		}

		// Check the staged devices if on the route toward an archive
		for (Iterator<String> iter = stagedDevices.iterator(); iter.hasNext(); ) {
			String device = iter.next();
			for ( String archive : archiveServers ) {
				if ( device.equals( DotcomConfig.getConfig().getServerNameTowards( archive ) ) ) {
					// Staged device is on the route to an archive
					if ( checkAndSetArchiveDeviceTo( study, storestate, archive ) ) {
						return;
					}
				}
			}
		}

		// No archive found so far, set the first archiveServer 
		for ( String archive : archiveServers ) {
			if ( checkAndSetArchiveDeviceTo( study, storestate, archive ) ) {
				return;
			}
		}
	}

%><%

	IMSession is = new IMSession(request, response, out);

	response.setContentType("text/plain");

	File tempDir = new File(IMEnvBase.getEnv("TMP") + "/correct_multihub");
	if (!tempDir.exists())
		tempDir.mkdir();

	MultipartRequest mr = new MultipartRequest(is, tempDir.getAbsolutePath());

/*
	problem: ha a nagyszulorol van redirectalva a hivas, az nem lesz benne a tablaban!
	try {
		Properties prpPeer = imagemed.jsputil.DotcomToolbox.checkClient(is, null, false);
		if (prpPeer != null) {
			String deviceID = prpPeer.getProperty("ID","");
			if ("".equals(deviceID)) {
				throw new Throwable();
			}
		}
	}
	catch (Throwable th) {
		out.println("ERROR: Unauthorized peer");
		return;
	}
*/
	
	String parStudy = mr.getParameter("cStudy");
	String parSession = mr.getParameter("cSession");
	String parUser = mr.getParameter("cUser");
	String parEndDevice = mr.getParameter( "cEndDevice" );

	boolean isMove = "true".equals(mr.getParameter("cMove"));
	boolean deleteExistingReports = "true".equals(mr.getParameter("cDelReport"));

	if (parStudy == null) {
		out.println("ERROR: Missing parameter: cStudy");
//		out.println(Helper.request2String2(request));
		return;
	}

	Helper.DEBUG( "PutObjects called, parEndDevice: " + parEndDevice );
	
	//redirect if needed
	if( DotcomConfig.selfDevice().isDbOnly() && parEndDevice != null && !"".equals( parEndDevice ) ) {
		String redirURL = RedirectToolbox.getURL( DotcomConfig.getConfig().getServerNameTowards( parEndDevice ), is.regIP, null );
		if( redirURL != null ) {
			redirURL += "/dotcom/putObjects.jsp";
			if( request.getQueryString() != null ) {
				redirURL += "?" + request.getQueryString();
			}
			
			Helper.DEBUG( "PutObjects redirect, URL: " + redirURL );

			response.sendRedirect(redirURL);
			return;
		}
	}

	if (parSession == null) {
		parSession = "" + System.currentTimeMillis() + "." + Helper.getCounter();
	}

	// container for folders to be created for shortcuts 
	Set<String> folders = new HashSet<String>();
	
	// working directory to untar files received from HUB(s)
	File workDir = new File( IMEnvBase.getEnv( "TMP" ) + "/chmput." + parSession );
	if ( ! workDir.mkdir() ) {
		out.println("ERROR: Working directory could not be created");
		return;
	}
	
	int numFiles = 0;
	try {
		numFiles = Integer.parseInt(mr.getParameter("cNumFiles"));
	}
	catch (Throwable th) {
	}

	// Command to untar files
	String[] tarCmd = new String[5];
	tarCmd[ 0 ] = "/bin/tar";
	tarCmd[ 1 ] = "xzf";
	tarCmd[ 2 ] = null; // tar filename will be set before calling tar
	tarCmd[ 3 ] = "-C";
	tarCmd[ 4 ] = null; // output directory will be set before calling tar
	
	
	// collect HUB(s) for log messages
	Vector<String> hubs = new Vector<String>();
	
	// command to put objects received from hubs to their final place
	Vector<String> cmd = new Vector<String>();
	cmd.add(IMEnvBase.getEnv("DOTCOM_HOME") + "/fixMultihub.sh");
	cmd.add("-s");
	cmd.add(parStudy);

	cmd.add( "-w" );
	cmd.add( workDir.getAbsolutePath() );
	
	if (parUser != null) {
		cmd.add("-u");
		cmd.add(parUser);
	}

	if (isMove) {
		cmd.add("-n");
	}
	
	if (deleteExistingReports) {
		cmd.add("-r");
	}
	
	if (numFiles > 0) {
		for (int i = 0; i < numFiles; i++) {
			String hub = mr.getParameter("cHub" + i);
			if (null == hub) {
				out.println("ERROR: Missing parameter: cHub" + i);
				return;
			}

			hubs.add(hub);
			String tarFile = mr.getParameter("cFile" + i);
			if (tarFile == null) {
				out.println("ERROR: Missing parameter: cFile" + i);
				return;
			}

			File outDir = new File( workDir, hub );
			if ( ! outDir.mkdir() ) {
				out.println("ERROR: Working directory could not be created");
				return;
			}

			// Untar tar files
			tarCmd[ 2 ] = tarFile;
			tarCmd[ 4 ] = outDir.getAbsolutePath();
			RunCommandResult tarRes = Helper.runCommandEx( tarCmd, null );
			if ( tarRes.getExitCode() != 0 ) {
				out.println("ERROR: " + tarFile + " could not be untarred");
				return;
			}

			// Read CONTENT
			IMProperties tarContent = new IMProperties();
			tarContent.recRead( outDir.getAbsolutePath() + "/CONTENT" );
			if ( tarContent.isEmpty() ) {
				out.println("ERROR: CONTENT file could not be read");
				return;
			}

			int numShortcuts = 0;
			try {
				numShortcuts = Integer.parseInt( tarContent.getProperty( "chmNumShortcuts", "0" ) );
			}
			catch ( Throwable th ) {
			}
			
			for ( int j = 0 ; j < numShortcuts ; j++ ) {
				String folder = tarContent.getProperty( "chmShortcut" + j + "_FOLDER", "" );
				if ( ! "".equals( folder ) ) {
					folders.add( folder );
				}
			}
			
			cmd.add(hub);
		}
	}

	// Check and create folders
	for ( Iterator<String> iter = folders.iterator() ; iter.hasNext() ; ) {
		String folder = iter.next();
		try {
			FolderToolbox.createFolder( is, folder, true );
		}
		catch (IMException imEx) {
			Helper.ERRORLOG("putObjects", "Exception in FolderToolbox.createFolder: " + Helper.throwable2String( imEx ) );
			out.println("ERROR: Folder " + folder + " could not be created" );
			return;
		}
	}
	
	
	log(parStudy, parUser, hubs, isMove, deleteExistingReports, null);

	try {
		// Check and Fix Archive Server settings
		checkAndFixArchiveServer(parStudy);
	}
	catch (Throwable th) {
		out.println("ERROR [" + th.toString() + "]");
		log(parStudy, parUser, hubs, isMove, deleteExistingReports, "An error occured while updating ArchiveServer attribute in storestate.rec");
		return;
	}

	for (Iterator<String> iter = cmd.iterator(); iter.hasNext(); ) {
		out.print("[");
		out.print(iter.next());
		out.print("] ");
	}
	out.println();

	RunCommandResult res = Helper.runCommandEx(cmd.toArray(new String[0]), m_env);
	out.println(res.getOutput());
	if (res.getExitCode() != 0) {
		out.println("ERROR [" + res.getExitCode() + "]: " + res.getError());
		log(parStudy, parUser, hubs, isMove, deleteExistingReports, "An error occured while storing objects received from other hub(s)");
		Helper.writeTextFile( workDir.getAbsolutePath() + "/done", "ERROR" );
		return;
	}

	Helper.writeTextFile( workDir.getAbsolutePath() + "/done", "DONE" );
	out.println("#RESULT: OK");

%>
