#!/bin/sh
#
# Component: dotcom
#
# Description:
#   Proliferates a file or directory within a repository in the dotcom.
#

. $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="proliferateResource[$$]"


function errorExit_() {
	echo "$@" >&2
	$LOGGER -p $LOG_FACILITY.err -t "$TAG" "$@"
	exit 1
}


#
# Usage
#
function usage_()
{
	if [ -n "$1" ] ; then
		echo $1
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "$1"
	fi
	cat <<-ENDOFUSAGE
		proliferateResource.sh -R <repository> -N <resource name> [-P <path>] [-D]
		       [-n] [-d <direction>] [-x <excepthost>]

		           -R <repository>    : name of the repository (environment variable name)
		           -N <resource name> : the name of the resource in the repository
		           -P <path>          : relative path to the file within the repository resource to proliferate
		                                if missing, then the entire resource will be proliferated
		           -D                 : proliferate the deletion of the resource or path
		           -n                 : do not proliferate to Database servers
		           -x <excepthost>    : don't send the file to this host
		           -d <direction>     : direction of proliferate {up|down|all|backup|all-backup}, default is all-backup
	ENDOFUSAGE

	[ -n "$1" ] && exit 1
	exit 0
} # usage


ProlifDirection="all"
ProlifExcept=""
ProlifToDbOnly="true"

RepoName=""
ResourceName=""
FilePath=""
FileType=""
IsDeleted="false"


while getopts ':DR:N:P:x:d:n' OPT; do
	case $OPT in
		D) IsDeleted="true" ;;
		R) RepoName="${OPTARG}" ;;
		N) ResourceName="${OPTARG}" ;;
		P) FilePath="${OPTARG}" ;;
		x) ProlifExcept="${OPTARG}" ;;
		d) ProlifDirection="${OPTARG}" ;;
		n) ProlifToDbOnly="false" ;;
		*) usage_ "Invalid Option: -${OPTARG}" ;;
	esac
done

[ -z "${RepoName}" ] && usage_ "Missing mandatory option -R <repository>"
[ -z "${ResourceName}" ] && usage_ "Missing mandatory option -N <resource name>"
RepoDir="${!RepoName}" 
[ ! -d "${RepoDir}" ] && errorExit_ "Repository '${RepoDir}' (${RepoName}) does not exist"

EntryLocation=$( ${REPOSITORY_HANDLER_HOME}/bin/locateResource "${RepoDir}" "${ResourceName}" )
LocRes=$?
if [ $(( ${LocRes} & 4 )) -eq 4 ] ; then
	# resource was not found in the repository
	# might be valid when resource is deleted (path empty and IsDeleted)
	if [ -n "${FilePath}" -o "${IsDeleted}" != "true" ] ; then
		errorExit_ "Resource '${ResourceName}' was not found in repository '${RepoDir}'"
	fi
elif [ ${LocRes} -ne 0 -o -z "${EntryLocation}" ] ; then
	errorExit_ "An error occurred while locating resource '${ResourceName}' in repository '${RepoDir}'"
fi

FullPath="${EntryLocation}"
if [ -n "${FilePath}" ] ; then
	FullPath="${EntryLocation}/${FilePath}"
fi


SELF_SERVER_ID=""
[ -f "$CONF/serverid.cfg" ] && . $CONF/serverid.cfg
if [ -z "$SELF_SERVER_ID" ] ; then
	SELF_SERVER_ID="unknown"
fi

ProlifFileName="${TMP}/prolifrepo.${SELF_SERVER_ID}.$(date +%Y%m%d%H%M%S).$$"

if [ "${IsDeleted}" = "true" ] ; then
	FileType="delete"

	[ -e "${FullPath}" ] && errorExit_ "File '${FullPath}' expected to be deleted but exists"

elif [ -f "${FullPath}" ] ; then
	FileType="file"

	cp -p "${FullPath}" "${ProlifFileName}"
	[ $? -ne 0 ] && errorExit_ "Could not copy '${FullPath}' to '${ProlifFileName}'"

elif [ -d "${FullPath}" ] ; then
	FileType="dir"

	tar -cf "${ProlifFileName}" -C "${FullPath}" .
	[ $? -ne 0 ] && errorExit_ "Could not tar '${FullPath}' to '${ProlifFileName}'"

	touch -r "${FullPath}" "${ProlifFileName}"

else
	errorExit_ "File '${FullPath}' does not exist or has an unsupported file type"
fi


Command="${DOTCOM_HOME}/proliferate.sh -f '${ProlifFileName}'"
[ -n "$ProlifDirection" ] && Command="${Command} -d '${ProlifDirection}'"
[ -n "$ProlifExcept" ] && Command="${Command} -x '${ProlifExcept}'"
[ "true" != "${ProlifToDbOnly}" ] && Command="${Command} -n"

Command="${Command} -r '${DOTCOM_HOME}/storeResource.sh'"

Command="${Command} -a -R -a '${RepoName}'"
Command="${Command} -a -N -a '${ResourceName}'"
Command="${Command} -a -T -a '${FileType}'"
[ -n "${FilePath}" ] && Command="${Command} -a -P -a '${FilePath}'"


$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "call proliferate.sh: '${Command}'"
$TASKD_HOME/createtaskfileapp -p "PROLIFERATE" -t "sh" -c "${Command}"

exit 0
