#!/bin/sh
#
# Component: dotcom
#
# Description:
#   Stores proliferated repository resource or a file within a resource.
#

. $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="storeResource[$$]"


function error_() {
	echo "$@" >&2
	$LOGGER -p $LOG_FACILITY.err -t "$TAG" "$@"
	return 0
}



function errorExit_() {
	error_ "$@"
	exit 1
}


#
# Usage
#
function usage_()
{
	if [ -n "$1" ] ; then
		echo $1
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "$1"
	fi
	cat <<- ENDOFUSAGE
		storeResource.sh -f <filename> -R <repository> -N <resource name> -T {file|dir|delete} [-P <path>] [-n] [-d <direction>] [-x <excepthost>]
	
		           -f <filename>        : name of the file stored (proliferated)
		           -R <repository>      : name of the repository (environment variable name)
		           -N <resource name>   : the name of the resource in the repository
		           -P <path>            : relative path to the file within the repository resource
		                                  if missing, then the entire resource was proliferated
		           -T {file|dir|delete} : the type of the received file/resource
		           -n                   : do not proliferate to Database servers
		           -x <excepthost>      : don't send the file to this host (Host ID (preferred) / IP address (obsolete))
		           -d <direction>       : direction of proliferation {up|down|all}, default is all
	ENDOFUSAGE

	[ -n "$1" ] && exit 1
	exit 0
} # usage



#
# $1 - the file to move to the repository
# $2 - temporary location on the mount point
# $3 - the final location
#
function copyFileToRepo_() {
	local Src="$1"
	local Temp="$2"
	local Dst="$3"

	[ -z "${Src}" -o -z "${Temp}" -o -z "${Dst}" ] && error_ "copyFileToRepo_(): invalid call" && return 1

	# Copy to the mount point first
	cp -p "${Src}" "${Temp}"
	if [ $? -ne 0 ] ; then
		[ -f "${Temp}" ] && rm "${Temp}"
		error_ "The file could not be copied to the repository mount point '${Temp}'"
		return 1
	fi

	# Rename on the same mount point
	mv -f -T "${Temp}" "${Dst}"
	if [ $? -ne 0 ] ; then
		[ -f "${Temp}" ] && rm "${Temp}"
		error_ "Moving the file to its final location '${Dst}' failed"
		return 1
	fi

	return 0;
}



#
# $1 - the directory to move to the repository
# $2 - temporary location on the mount point
# $3 - the final location
# $4 - backup for the original directory (if any)
#
function moveDirToRepo_() {
	local Src="$1"
	local Temp="$2"
	local Dst="$3"
	local Backup="$4"

	local TargetDir

	local Res=1

	[ -z "${Src}" -o -z "${Temp}" -o -z "${Dst}" ] && error_ "moveDirToRepo_(): invalid call" && return 1

	# mv the new directory to the same mount first
	mv -T "${Src}" "${Temp}"
	if [ $? -ne 0 ] ; then
		error_ "The directory could not be moved to the repository mount point '${Temp}'"
		return 1
	fi

	if [ -e "${Dst}" ] ; then 	
		# XXX : Not atomic, but found no better option
		mv -T "${Dst}" "${Backup}" && mv -T "${Temp}" "${Dst}"
		Res=$?

		if [ $Res -ne 0 ] ; then
			mv -T "${Backup}" "${Dst}"
		fi
	else
		TargetDir=$( dirname "${Dst}" )
		[ ! -e "${TargetDir}" ] && mkdir -p "${TargetDir}"
		mv -T "${Temp}" "${Dst}"
		Res=$?
	fi

	if [ $Res -ne 0 ] ; then
		[ -e "${Temp}" ] && rm -fr "${Temp}" 
		error_ "The resource could not be moved to its location '${Dst}'"
		return 1
	fi


	[ -e "${Backup}" ] && rm -fr "${Backup}"

	return 0;
}


#
# $1 - The file to untar
# $2 - path to the extracted directory (will be created)
#
function unTarTo_() {
	local TarFile="$1"
	local ExtractDir="$2"

	mkdir "${ExtractDir}"
	if [ $? -ne 0 ] ; then
		error_ "Working directory '${ExtractDir}' could not be created"
		return 1
	fi

	tar -xf "${TarFile}" -C "${ExtractDir}"
	if [ $? -ne 0 ] ; then
		[ -e "${ExtractDir}" ] && rm -fr "${ExtractDir}"
		error_ "Could not extract '${TarFile}'"
		return 1
	fi

	return 0
}




#
# Stores the received file as a file resource
#
# $1 - The file received
# $2 - Path to the repository 
# $3 - Resource Name
#
function storeFileResource_() {
	local File="$1"
	local Repository="$2"
	local Name="$3"

	local Location
	local HashDir
	local TempName="applyRepo.$(date +%Y%m%d%H%M%S).$$.${Name}"
	local IsNew="false"

	Location=$( ${REPOSITORY_HANDLER_HOME}/bin/locateResource "${Repository}" "${Name}" 2>/dev/null )
	if [ -z "${Location}" ] ; then
			Location=$( ${REPOSITORY_HANDLER_HOME}/bin/createLocation "${Repository}" "${Name}" 2>/dev/null )
			if [ $? -ne 0 -o -z "${Location}" ] ; then
				echo "Repository location could not be created for '${Name}'" >&2
				return 1
			fi
			IsNew="true"
	fi

	HashDir=$( dirname "${Location}" )
	copyFileToRepo_ "${File}" "${HashDir}/${TempName}" "${Location}"
	if [ $? -ne 0 ] ; then
		[ -f "${HashDir}/${TempName}" ] && rm "${HashDir}/${TempName}"
		[ "${IsNew}" = "true" ] && ${REPOSITORY_HANDLER_HOME}/bin/removeLocation "${Repository}" "${Name}"
		return 1
	fi

	return 0;
}




#
# Stores the received file as a directory resource
#
# $1 - The file received (expected to be a tar file)
# $2 - Path to the repository 
# $3 - Resource Name
#
function storeDirResource_() {
	local File="$1"
	local Repository="$2"
	local Name="$3"

	local Location
	local HashDir
	local TempName="applyRepo.$(date +%Y%m%d%H%M%S).$$.${Name}"
	local WorkDir="${TMP}/${TempName}"
	local BackupName="backup.$$.${Name}"
	local IsNew="false"

	unTarTo_ "${File}" "${WorkDir}" || return 1

	Location=$( ${REPOSITORY_HANDLER_HOME}/bin/locateResource "${Repository}" "${Name}" 2>/dev/null )
	if [ -z "${Location}" ] ; then
			Location=$( ${REPOSITORY_HANDLER_HOME}/bin/createDirectory "${Repository}" "${Name}" 2>/dev/null )
			if [ $? -ne 0 -o -z "${Location}" ] ; then
				error_ "Repository location could not be created for '${Name}'"
				return 1
			fi
			IsNew="true"
	fi

	# mv the new directory to the same mount first
	HashDir=$( dirname "${Location}" )

	moveDirToRepo_ "${WorkDir}" "${HashDir}/${TempName}" "${Location}" "${BackupName}"
	if [ $? -ne 0 ] ; then
		[ -e "${WorkDir}" ] && rm -fr "${WorkDir}"
		[ -e "${HashDir}/${TempName}" ] && rm -fr "${HashDir}/${TempName}"
		[ "${IsNew}" = "true" ] && ${REPOSITORY_HANDLER_HOME}/bin/removeLocation "${Repository}" "${Name}" 2>/dev/null
		return 1
	fi

	# These should have already gone, but for safety
	[ -e "${HashDir}/${TempName}" ] && rm -fr "${HashDir}/${TempName}"
	[ -e "${WorkDir}" ] && rm -fr "${WorkDir}"

	return 0;
}


#
# Stores the received file as a directory resource
#
# $1 - The file received (expected to be a tar file)
# $2 - Path to the repository 
# $3 - Resource Name
# $4 - Path within the resource directory
# $5 - File type (file/dir)
#
function storeFile_() {
	local File="$1"
	local Repository="$2"
	local Name="$3"
	local Path="$4"
	local Type="$5"

	local TargetName=$( basename "${Path}" )
	local TempName="applyRepo.$(date +%Y%m%d%H%M%S).$$.${TargetName}"
	local WorkDir="${TMP}/${TempName}"
	local BackupName="backup.$$.${TargetName}"

	local Location
	local HashDir
	local TargetDir

	Location=$( ${REPOSITORY_HANDLER_HOME}/bin/createDirectory "${Repository}" "${Name}" 2>/dev/null )
	if [ $? -ne 0 -o -z "${Location}" ] ; then
		error_ "Resource '${Name}' could not be found/created in repository '${Repository}'"
		return 1
	fi

	HashDir=$( dirname "${Location}" )


	if [ "${Type}" = "file" ] ; then

		# TODO: debug only, to be removed
		stat "${File}"
		stat "${Location}/${Path}"

		if [ ! "${File}" -nt "${Location}/${Path}" ] ; then
			error_ "A newer file was found in the repository '${Location}/${Path}'"
			return 2
		fi

		TargetDir=$( dirname "${Location}/${Path}" )
		[ ! -e "${TargetDir}" ] && mkdir -p "${TargetDir}"

		copyFileToRepo_ "${File}" "${HashDir}/${TempName}" "${Location}/${Path}"
		if [ $? -ne 0 ] ; then
			[ -f "${HashDir}/${TempName}" ] && rm "${HashDir}/${TempName}"
			return 1
		fi

		return 0
	elif [ "${Type}" = "dir" ] ; then
		# Last modified times are not checked, because directory time does not change when
		# - adding modifying files in the directory, or when
		# - adding/removing files to/from a sub-directory

		unTarTo_ "${File}" "${WorkDir}" || return 1
		moveDirToRepo_ "${WorkDir}" "${HashDir}/${TempName}" "${Location}/${Path}" "${HashDir}/${BackupName}"
		if [ $? -ne 0 ] ; then
			[ -e "${WorkDir}" ] && rm -fr "${WorkDir}"
			[ -e "${HashDir}/${TempName}" ] && rm -fr "${HashDir}/${TempName}"
			return 1
		fi

		# These should have already gone, but for safety
		[ -e "${HashDir}/${TempName}" ] && rm -fr "${HashDir}/${TempName}"
		[ -e "${WorkDir}" ] && rm -fr "${WorkDir}"

		return 0
	fi

	error_ "Unrecognized file type '${Type}'"
	return 1
}


#
# Deletes resource or a file
#
# $1 - Path to the repository 
# $2 - Resource Name
# $3 - Path within the resource directory
#
function deleteFile_() {
	local Repository="$1"
	local Name="$2"
	local Path="$3"

	local Res
	local Location

	if [ -z "${Path}" ] ; then
		${REPOSITORY_HANDLER_HOME}/bin/removeResource "${Repository}" "${Name}" 2>/dev/null
		Res=$?
		# successful delete
		[ $Res -eq 0 ] && return 0
		# did not exist
		[ $(( $Res & 4 )) -eq 4 ] && return 0

		error_ "Could not delete resource '${Name}'"
		return 1
	fi

	Location=$( ${REPOSITORY_HANDLER_HOME}/bin/locateResource "${Repository}" "${Name}" 2>/dev/null )
	Res=$?
	# the resource does not exist
	[ $(( $Res & 4 )) -eq 4 ] && return 0

	if [ $Res -ne 0 -o -z "${Location}" ] ; then
		error_ "Resource '${Name}' could not be found in repository '${Repository}'"
		return 1
	fi

	if [ -e "${Location}/${Path}" ] ; then
		rm -fr "${Location}/${Path}"
		[ $? -ne 0 ] && error_ "File '${Location}/${Path}' could not be deleted" && return 1
	fi

	return 0
}



ProlifDirection="all"
ProlifExcept=""
ProlifToDbOnly="true"

RepoName=""
ResourceName=""
FilePath=""
FileType=""

TheFile=""

while getopts ':R:N:P:T:f:x:d:n' OPT; do
	case $OPT in
		R) RepoName="${OPTARG}" ;;
		N) ResourceName="${OPTARG}" ;;
		P) FilePath="${OPTARG}" ;;
		T) FileType="${OPTARG}" ;;
		f) TheFile="${OPTARG}" ;;
		x) ProlifExcept="${OPTARG}" ;;
		d) ProlifDirection="${OPTARG}" ;;
		n) ProlifToDbOnly="false" ;;
		*) usage_ "Invalid Option: -${OPTARG}" ;;
	esac
done

[ -z "${TheFile}" ] && usage_ "Missing Mandatory option -f"
[ ! -f "${TheFile}" ] && [ "${FileType}" != "delete" ] && errorExit_ "File '${TheFile}' does not exist"

[ -z "${RepoName}" ] && usage_ "Missing mandatory option -R <repository>"
[ -z "${ResourceName}" ] && usage_ "Missing mandatory option -N <resource name>"
RepoDir="${!RepoName}"
[ ! -d "${RepoDir}" ] && errorExit_ "Repository '${RepoDir}' (${RepoName}) does not exist"

[ -z "${FileType}" ] && usage_ "Missing mandatory option -T <file type>"


Result=1

if [ "${FileType}" = "delete" ] ; then

	deleteFile_ "${RepoDir}" "${ResourceName}" "${FilePath}"
	Result=$?

elif [ -n "${FilePath}" ] ; then

	storeFile_ "${TheFile}" "${RepoDir}" "${ResourceName}" "${FilePath}" "${FileType}"
	Result=$?

elif [ "${FileType}" = "file" ] ; then

	storeFileResource_ "${TheFile}" "${RepoDir}" "${ResourceName}"
	Result=$?

elif [ "${FileType}" = "dir" ] ; then

	storeDirResource_ "${TheFile}" "${RepoDir}" "${ResourceName}"
	Result=$?

else

	error_ "Unrecognized file type: '${FileType}'"

fi


if [ $Result -eq 0 ] ; then
	Command="${DOTCOM_HOME}/proliferate.sh -f '${TheFile}'"
	[ -n "$ProlifDirection" ] && Command="${Command} -d '${ProlifDirection}'"
	[ -n "$ProlifExcept" ] && Command="${Command} -x '${ProlifExcept}'"
	[ "true" != "${ProlifToDbOnly}" ] && Command="${Command} -n"

	Command="${Command} -r '${DOTCOM_HOME}/storeResource.sh'"
	Command="${Command} -a -R -a '${RepoName}'"
	Command="${Command} -a -N -a '${ResourceName}'"
	Command="${Command} -a -T -a '${FileType}'"
	[ -n "${FilePath}" ] && Command="${Command} -a -P -a '${FilePath}'"

	$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "call proliferate.sh: '${Command}'"
	$TASKD_HOME/createtaskfileapp -p "PROLIFERATE" -t "sh" -c "${Command}"

	exit 0
else
	$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "store failed; deleting ${TheFile}"
	rm "${TheFile}"
fi


exit $Result

